class ImportXmlJob < ApplicationJob
  queue_as :default

  # xml_root_node_name => parse_info
  def node_map
    @_node_map ||= {
        "podata" => {
            "item_type" => "poheader",
            "class" => PoHeader
        },
        "storejbdata" => {
            "item_type" => "storejbtx",
            "class" => StoreJobDatum
        },
        "storededata" => {
            "item_type" => "storedetx",
            "class" => StoreJobDatum
        },
        "lensdata" => {
            "item_type" => "lensdetail",
            "class" => Lens
        },
        "stxdata" => {
            "item_type" => "stxdetail",
            "class" => LensTransaction
        },
        "orderdata" => {
            "item_type" => "orderdetail",
            "class" => Order
        },
        "ccstxdata" => {
            "item_type" => "ccstxdetail",
            "class" => CorporateConsolidatedStockTransaction
        },
    }
  end

  def delay
    60
  end

  def perform(file_path, last_size)
    ActiveRecord::Base.connection_pool.with_connection do
      begin
        @file_path = file_path
        Rails.logger.info { {action: 'import', msg: "Checking if file is ready to process", file: @file_path} }
        if File.exist?(@file_path)
          guard_file = @file_path.sub(/#{File.extname(@file_path)}$/, '') + '.guard'
          if File.exist?(guard_file)
            Rails.logger.info { {action: 'import', msg: "Guard file exists, retrying later", file: @file_path} }
            ImportXmlJob.set(wait: delay.seconds).perform_later(@file_path, File.size(@file_path))
          elsif (Time.now - File.atime(@file_path)) <= delay
            Rails.logger.info { {action: 'import', msg: "File was modified within #{delay} seconds (atime), retrying later", file: @file_path} }
            ImportXmlJob.set(wait: delay.seconds).perform_later(@file_path, File.size(@file_path))
          elsif (Time.now - File.mtime(@file_path)) <= delay
            Rails.logger.info { {action: 'import', msg: "File was modified within #{delay} seconds (mtime), retrying later", file: @file_path} }
            ImportXmlJob.set(wait: delay.seconds).perform_later(@file_path, File.size(@file_path))
          elsif (Time.now - File.ctime(@file_path)) <= delay
            Rails.logger.info { {action: 'import', msg: "File was modified within #{delay} seconds (ctime), retrying later", file: @file_path} }
            ImportXmlJob.set(wait: delay.seconds).perform_later(@file_path, File.size(@file_path))
          elsif last_size != File.size(@file_path)
            Rails.logger.info { {action: 'import', msg: "File was modified within #{delay} seconds (size), retrying later", file: @file_path} }
            ImportXmlJob.set(wait: delay.seconds).perform_later(@file_path, File.size(@file_path))
          else
            counter = error_counter = global_counter = 0
            File.open(@file_path, "r") do |f|
              if f.flock(File::LOCK_EX | File::LOCK_NB)
                begin
                  Rails.logger.info { {action: 'import', msg: "Processing", file: @file_path} }
                  global_values = {}
                  model = nil
                  target_tag = nil
                  Nokogiri::XML::Reader(f).each do |node|
                    if node.node_type == Nokogiri::XML::Reader::TYPE_ELEMENT
                      if %w(lmssource lenslmssource stxlmssource polmssource ccstxlmssource orderlmssource jblmssource delmssource).include?(node.name)
                        Rails.logger.debug { {action: 'import', msg: "Identified LMS #{node.inner_xml}", file: @file_path} }
                        global_values["lmssource"] = node.inner_xml
                      elsif node_map.has_key?(node.name)
                        Rails.logger.info { {action: 'import', msg: "Identified import type #{node.name}", file: @file_path} }
                        target_tag = node_map[node.name]["item_type"]
                        model = node_map[node.name]["class"]
                        counter = 0
                      elsif target_tag == node.name
                        begin
                          model.import_from_hash(global_values.merge(Hash.from_xml(node.outer_xml)[target_tag])).save
                          counter += 1
                          global_counter += 1
                          Rails.logger.info { {action: 'import', msg: "Imported #{counter} #{target_tag} so far...", file: @file_path} } if counter > 0 && ((counter % 50) == 0)
                        rescue StandardError => err
                          error_counter += 1
                          Rails.logger.warn { {action: 'import', msg: "Failed importing #{target_tag} node with details #{node.outer_xml}", file: @file_path} }
                          begin
                            FileUtils.mkdir_p(FileMonitorConfig.instance.error_directory)
                            File.write(error_details_filename, "#{node.outer_xml}\n\n\n", mode: 'a')
                            File.write(error_details_filename, err.message, mode: 'a')
                          rescue StandardError => write_err
                            Rails.logger.warn { {action: 'import', msg: "Failed writing to #{error_details_filename}", file: @file_path} }
                          end
                        end
                      end
                    end
                  end
                  Rails.logger.info { {action: 'import', msg: "Finished (processed #{global_counter} top-level records) with #{error_counter} errors", file: @file_path} }
                rescue StandardError => err
                  Rails.logger.error { {action: 'import', msg: "Exception during import", file: @file_path} }
                end
              end
            end
            if error_counter > 0
              move_to_error!
            else
              move_to_archive!
            end
          end
        else
          Rails.logger.debug { {action: 'import', msg: "File does not appear to exist.  It may have already been processed.", file: @file_path} }
        end
      rescue StandardError => err
        Rails.logger.error { {msf: 'import error', file: @file_path} }
        sleep(30)
        FileUtils.touch(file_path)
      end
    end
  end

  def file_proc_timestamp
    @fpts ||= RxModule::Time.file_timestamp
  end

  def error_details_filename
    f_name = @file_path.gsub(File.dirname(@file_path), '').gsub('/', '').gsub('\\', '').gsub('.xml', '')
    File.join(FileMonitorConfig.instance.error_directory, "#{file_proc_timestamp}.#{f_name}.details.txt")
  end

  def error_raw_filename
    f_name = @file_path.gsub(File.dirname(@file_path), '').gsub('/', '').gsub('\\', '')
    File.join(FileMonitorConfig.instance.error_directory, "#{file_proc_timestamp}.#{f_name}")
  end

  def archive_filename
    f_name = @file_path.gsub(File.dirname(@file_path), '').gsub('/', '').gsub('\\', '')
    File.join(FileMonitorConfig.instance.archive_directory, "#{file_proc_timestamp}.#{f_name}")
  end

  def move_to_error!
    begin
      FileUtils.mkdir_p(FileMonitorConfig.instance.error_directory)
      if File.exist?(@file_path)
        FileUtils.move(@file_path, error_raw_filename)
        Rails.logger.info { {action: 'import', file: @file_path, msg: "Moved #{@file_path} to #{error_raw_filename}"} }
      end
    rescue StandardError => moveError
      Rails.logger.error { {action: 'import', file: @file_path, msg: "Failed to move to error directory"} }
    end
  end

  def move_to_archive!
    begin
      if FileMonitorConfig.instance.archive_directory
        FileUtils.mkdir_p(FileMonitorConfig.instance.archive_directory)
        if File.exist?(@file_path)
          FileUtils.move(@file_path, archive_filename)
          Rails.logger.info { {action: 'import', file: @file_path, msg: "Moved #{@file_path} to #{archive_filename}"} }
        end
      end
    rescue StandardError => moveError
      Rails.logger.error { {action: 'import', file: @file_path, msg: "Failed to move to archive directory"} }
    end
  end
end
