# == Schema Information
#
# Table name: corporate_consolidated_stock_transactions
#
#  id                              :bigint           not null, primary key
#  ccstxbeginningperpetualbalance  :decimal(12, 2)
#  ccstxcalcendingperpetualbalance :decimal(12, 2)
#  ccstxdate                       :date             not null
#  ccstxendingperpetualbalance     :decimal(12, 2)
#  ccstxlenswarehouseid            :string
#  ccstxopenpurchases              :decimal(12, 2)
#  ccstxperpetualbalance           :decimal(12, 2)
#  ccstxperpetualbalanceadjust     :decimal(12, 2)
#  ccstxtotaladjustcost            :decimal(12, 2)
#  ccstxtotaladjustqty             :integer
#  ccstxtotalcountvariance         :integer
#  ccstxtotalcountvariancecost     :decimal(12, 2)
#  ccstxtotalcyclecountcost        :decimal(12, 2)
#  ccstxtotalcyclecountqty         :integer
#  ccstxtotallensadjustments       :decimal(12, 2)
#  ccstxtotalpocost                :decimal(12, 2)
#  ccstxtotalpoqty                 :integer
#  ccstxtotalqty                   :integer
#  ccstxtotalreceivecost           :decimal(12, 2)
#  ccstxtotalreceiveqty            :integer
#  ccstxtotalreturncost            :decimal(12, 2)
#  ccstxtotalreturnqty             :integer
#  ccstxtotalusagecost             :decimal(12, 2)
#  ccstxtotalusageqty              :integer
#  ccstxtotalwastagecost           :decimal(12, 2)
#  ccstxtotalwastageqty            :integer
#  is_placeholder                  :boolean
#  lmssource                       :string           not null
#  created_at                      :datetime         not null
#  updated_at                      :datetime         not null
#
# Indexes
#
#  ccstx_unique_id                                               (ccstxlenswarehouseid,ccstxdate,lmssource) UNIQUE
#  index_corporate_consolidated_stock_transactions_on_ccstxdate  (ccstxdate)
#
class CorporateConsolidatedStockTransaction < ApplicationRecord
  class << self
    def import_from_hash(h)
      h.merge!(h.delete("ccstx") || {})
      h['ccstxlenswarehouseid'] ||= h.delete('lenswarehouseid')
      h['is_placeholder'] = false unless h.has_key? 'is_placeholder'
      super(h)
    end

    def conflict_target
      [:ccstxlenswarehouseid, :ccstxdate, :lmssource]
    end
  end

  compute_on_save(:ccstxbeginningperpetualbalance) do |ccst|
    most_recent_import = where(lmssource: ccst[:lmssource], ccstxlenswarehouseid: ccst[:ccstxlenswarehouseid])
                             .where(get_arel('ccstxdate', false).lt(ccst[:ccstxdate]))
                             .where(get_arel('is_placeholder', false).eq(false))
                             .order(ccstxdate: :desc)
                             .first
    if most_recent_import && !ccst['is_placeholder']
      most_recent_import[:ccstxdate].upto(ccst[:ccstxdate]) do |placeholder|
        unless [most_recent_import[:ccstxdate], ccst[:ccstxdate]].include?(placeholder)
          Rails.logger.debug { {action: 'import', msg: "stub - #{ccst[:ccstxlenswarehouseid]}, #{placeholder}"} }
          import_from_hash({'lmssource' => ccst[:lmssource],
                            'ccstxlenswarehouseid' => ccst[:ccstxlenswarehouseid],
                            'ccstxdate' => placeholder,
                            'ccstxperpetualbalance' => most_recent_import[:ccstxperpetualbalance],
                            'ccstxendingperpetualbalance' => most_recent_import[:ccstxendingperpetualbalance],
                            'is_placeholder' => true
                           }).save
        end
      end
    end
    most_recent_import&.[](:ccstxperpetualbalance)
  end

  compute_on_save(:ccstxcalcendingperpetualbalance) do |ccst|
    [:ccstxbeginningperpetualbalance, :ccstxtotalreceivecost, :ccstxtotalusagecost, :ccstxtotalwastagecost, :ccstxtotalcyclecountcost, :ccstxtotalreturncost, :ccstxtotaladjustcost].reduce(0) do |sum, col|
      sum + Float(ccst[col] || 0)
    end
  end

  reportable("corporate_consolidated_stock_transaction", "Corporate Consolidated Stock Transactions", "Report on corporate consolidated stock transactions", {
=begin
      lmssource: {
          name: "Lab Management System",
          shortname: "LMS",
          desc: "The lab management system that produced the stock transaction"
      },
=end
  ccstxlenswarehouseid: {
      name: "Warehouse ID",
      shortname: "Warehouse",
      desc: "The warehouse that produced the stock transaction"
  },
  ccstxdate: {
      name: "Transaction Date",
      shortname: "Date",
      desc: "The date of the stock transaction"
  },
  ccstxtotalqty: {
      name: "Total Quantity",
      shortname: "Total Qty",
      desc: "Total consolidated transaction quantity"
  },
  ccstxbeginningperpetualbalance: {
      name: "Beginning Perpetual Balance",
      desc: "Beginning perpetual balance"
  },
  ccstxtotalreceiveqty: {
      name: "Total Received Quantity",
      shortname: "Received Qty",
      desc: "Total consolidated received quantity"
  },
  ccstxtotalreceivecost: {
      name: "Total Received Cost",
      shortname: "Received Cost",
      desc: "Total consolidated received cost"
  },
  ccstxtotalusageqty: {
      name: "Total Usage Quantity",
      shortname: "Usage Qty",
      desc: "Total consolidated usage quantity"
  },
  ccstxtotalusagecost: {
      name: "Total Usage Cost",
      shortname: "Total Usage Cost",
      desc: "Total consolidated usage cost"
  },
  ccstxtotalwastageqty: {
      name: "Total Wastage Quantity",
      shortname: "Wastage Qty",
      desc: "Total consolidated wastage quantity"
  },
  ccstxtotalwastagecost: {
      name: "Total Wastage Cost",
      shortname: "Wastage Cost",
      desc: "Total consolidated wastage cost"
  },
  ccstxtotalreturnqty: {
      name: "Total Return Quantity",
      shortname: "Return Qty",
      desc: "Total consolidated return quantity"
  },
  ccstxtotalreturncost: {
      name: "Total Return Cost",
      shortname: "Return Cost",
      desc: "Total consolidated return cost"
  },
  ccstxtotaladjustqty: {
      name: "Total Adjust Quantity",
      shortname: "Adjust Qty",
      desc: "Total consolidated adjust quantity"
  },
  ccstxtotaladjustcost: {
      name: "Total Adjust Cost",
      shortname: "Adjust Cost",
      desc: "Total consolidated adjust cost"
  },
  ccstxtotalcountvariance: {
      name: "Total Count Variance",
      shortname: "Count Variance",
      desc: "Total consolidated count variance"
  },
  ccstxtotalcountvariancecost: {
      name: "Total Count Variance Cost",
      shortname: "Count Variance Cost",
      desc: "Total consolidated count variance cost"
  },
  ccstxtotalpoqty: {
      name: "Total Purchase Order Quantity",
      shortname: "PO Qty",
      desc: "Total consolidated purchase order quantity"
  },
  ccstxtotalpocost: {
      name: "Total Purchase Order Cost",
      shortname: "PO Cost",
      desc: "Total consolidated purchase order cost"
  },
  ccstxtotalcyclecountqty: {
      name: "Total Cycle Count Quantity",
      shortname: "Cycle Count Qty",
      desc: "Total consolidated cycle count quantity"
  },
  ccstxtotalcyclecountcost: {
      name: "Total Cycle Count Cost",
      shortname: "Cycle Count Cost",
      desc: "Total consolidated cycle count cost"
  },
  ccstxperpetualbalance: {
      name: "Perpetual Balance",
      desc: "Perpetual Balance"
  },
  ccstxopenpurchases: {
      name: "Open Purchases",
      desc: "Open Purchases"
  },
  ccstxperpetualbalanceadjust: {
      name: "Perpetual Balance Adjust",
      desc: "Perpetual Balance Adjust"
  },
  ccstxtotallensadjustments: {
      name: "Total Lens Adjustments",
      desc: "Lens Adjustments"
  },
  ccstxendingperpetualbalance: {
      name: "Ending Perpetual Balance",
      desc: "Ending Perpetual Balance"
  },
  ccstxcalcendingperpetualbalance: {
      name: "Calculated Ending Perpetual Balance",
      desc: "Calculated Ending Perpetual Balance"
  },
  })
end
