# == Schema Information
#
# Table name: lenses
#
#  id                       :bigint           not null, primary key
#  lensadd                  :decimal(7, 3)
#  lensbackcurve            :float
#  lensbackcurveradius      :float
#  lensbase                 :decimal(7, 3)
#  lensblkcehi              :float
#  lensblkd                 :float
#  lensblkdm                :float
#  lensblkrd                :float
#  lensbowldiameter         :float
#  lenscarriercurve         :float
#  lenscarriercurveradius   :float
#  lenscenterthickness      :float
#  lensclass                :string
#  lenscylinder             :decimal(7, 3)
#  lensdescription          :string           not null
#  lensdiameter             :float
#  lensfrontcurve           :float
#  lensfrontcurveradius     :float
#  lenskprva                :float
#  lenskprvm                :float
#  lenslblhi                :float
#  lensmanufacturercode     :string
#  lensmanufacturerdesc     :string
#  lensmaterialcode         :string
#  lensmaterialdesc         :string
#  lensnetitem              :boolean
#  lensopc                  :string
#  lensopcleft              :string
#  lensopcright             :string
#  lenspolarizedcurve       :float
#  lenspolarizedcurveradius :float
#  lenspolarizedmmfromfront :float
#  lensrightleftitem        :boolean
#  lensrngh                 :float
#  lenssagreading           :float
#  lenssegdepth             :float
#  lenssegmentthickness     :float
#  lenssegwidth             :float
#  lenssphere               :decimal(7, 3)
#  lensstylecode            :string
#  lensstyledesc            :string
#  lensstyletype            :string
#  lensthicknesscode        :string
#  lenstintcode             :string
#  lenstintdesc             :string
#  lenstreatcode            :string
#  lenstreatdesc            :string
#  lensupperadd             :decimal(7, 3)
#  lmssource                :string           not null
#  created_at               :datetime         not null
#  updated_at               :datetime         not null
#
# Indexes
#
#  index_lenses_on_lensdescription_and_lmssource  (lensdescription,lmssource) UNIQUE
#  index_lenses_on_lensmanufacturercode           (lensmanufacturercode)
#  index_lenses_on_lensmaterialcode               (lensmaterialcode)
#  index_lenses_on_lensopc                        (lensopc)
#  index_lenses_on_lensopcleft                    (lensopcleft)
#  index_lenses_on_lensopcright                   (lensopcright)
#  index_lenses_on_lensstylecode                  (lensstylecode)
#  index_lenses_on_lenstreatcode                  (lenstreatcode)
#
class Lens < ApplicationRecord
  has_many :lens_warehouse_data, dependent: :destroy
  has_many :lens_transactions, dependent: :nullify
  has_many :po_lines, dependent: :nullify

  class << self
    def import_from_hash(h)
      h.merge!(h.delete("lenscharacteristics") || {})
      h.merge!(h.delete("lenstechnical") || {})
      h['lenscylinder'] = h['lenscylinder'].to_f if h['lenscylinder']
      h['lenssphere'] = h['lenssphere'].to_f if h['lenssphere']
      h['lensbase'] = h['lensbase'].to_f if h['lensbase']
      h['lensadd'] = h['lensadd'].to_f if h['lensadd']
      h['lensupperadd'] = h['lensupperadd'].to_f if h['lensupperadd']
      warehouses = clean_multi_relation(h.delete("warehouse"))
      record = super(h)
      record.save
      warehouses.each do |w|
        w['lens_id'] = record.id
        LensWarehouseDatum.import_from_hash(w).save
      end
      record
    end

    def conflict_target
      [:lensdescription, :lmssource]
    end

    def append_on_import?(assoc)
      assoc&.to_s == "lens_warehouse_data"
    end
  end

  reportable("lens", "Lenses", "Report on lenses", {
=begin
      lmssource: {
          name: "Lab Management System",
          shortname: "LMS",
          desc: "The lab management system that produced the lens data"
      },
=end
  lensdescription: {
      name: "Description",
      desc: "Unique lens description"
  },
  lensmanufacturercode: {
      name: "Manufacturer Code",
      shortname: "Manufacturer",
      desc: "The manufacturer that produces the lens"
  },
  lensmanufacturerdesc: {
      name: "Manufacturer Description",
      shortname: "Manufacturer",
      desc: "The manufacturer that produces the lens"
  },
  lensstylecode: {
      name: "Style Code",
      shortname: "Style",
      desc: "The style of the lens"
  },
  lensstyledesc: {
      name: "Style Description",
      shortname: "Style",
      desc: "The style of the lens"
  },
  lensstyletype: {
      name: "Style Type",
      shortname: "Style Type",
      desc: "The style of the lens"
  },
  lensmaterialcode: {
      name: "Material Code",
      shortname: "Material",
      desc: "The material of the lens"
  },
  lensmaterialdesc: {
      name: "Material Description",
      shortname: "Material",
      desc: "The material of the lens"
  },
  lenstintcode: {
      name: "Tint Code",
      shortname: "Tint",
      desc: "The tint of the lens"
  },
  lenstintdesc: {
      name: "Tint Description",
      shortname: "Tint",
      desc: "The tint of the lens"
  },
  lenstreatcode: {
      name: "Treatment Code",
      shortname: "Treatment",
      desc: "The treatment of the lens"
  },
  lenstreatdesc: {
      name: "Treatment Description",
      shortname: "Treatment",
      desc: "The treatment of the lens"
  },
  lensclass: {
      name: "Class",
      desc: "The class of the lens"
  },
  lenssegwidth: {
      name: "Segment Width",
      shortname: "Seg Width",
      desc: "The width of the segment on the lens"
  },
  lenssegdepth: {
      name: "Segment Depth",
      shortname: "Seg Depth",
      desc: "The depth of the segment on the lens"
  },
  lenssphere: {
      name: "Sphere Power",
      shortname: "Sphere",
      desc: "The sphere power of the lens",
      format:{
          explicit_plus: true,
          zero_pad: 4,
          decimals: 0,
      }
  },
  lenscylinder: {
      name: "Cylinder Power",
      shortname: "Cyl",
      desc: "The cylinder power of the lens",
      format:{
          explicit_plus: true,
          zero_pad: 4,
          decimals: 0,
      }
  },
  lensbase: {
      name: "Base",
      desc: "The base of the lens",
      format:{
          explicit_plus: true,
          zero_pad: 4,
          decimals: 0,
      }
  },
  lensadd: {
      name: "Add Power",
      shortname: "Add",
      desc: "The add power of the lens",
      format:{
          explicit_plus: false,
          zero_pad: 4,
          decimals: 0,
      }
  },
  lensupperadd: {
      name: "Upper Add Power",
      shortname: "Upper Add",
      desc: "The upper add power of the lens",
      format:{
          explicit_plus: false,
          zero_pad: 4,
          decimals: 0,
      }
  },
  lensdiameter: {
      name: "Diameter",
      shortname: "Diam",
      desc: "The diameter of the lens"
  },
  lensthicknesscode: {
      name: "Thickness Code",
      shortname: "Thickness",
      desc: "The thickness of the lens"
  },
  lensopc: {
      name: "OPC",
      desc: "The opc of the lens"
  },
  lensopcright: {
      name: "Right OPC",
      shortname: "R.OPC",
      desc: "The opc of the right lens"
  },
  lensopcleft: {
      name: "Left OPC",
      shortname: "L.OPC",
      desc: "The opc of the left lens"
  },
  lensnetitem: {
      name: "Net Item?",
      shortname: "Net Item",
      desc: "Indicates if the lens is a net item"
  },
  lensrightleftitem: {
      name: "Right/Left Item?",
      shortname: "R/L Item",
      desc: "Indicates if the lens is a right/left item"
  },
  lensfrontcurve: {
      name: "Front Curve",
      desc: "The front curve of the lens"
  },
  lensfrontcurveradius: {
      name: "Front Curve Radius",
      desc: "The front curve radius of the lens"
  },
  lensbackcurve: {
      name: "Back Curve",
      desc: "The back curve of the lens"
  },
  lensbackcurveradius: {
      name: "Back Curve Radius",
      desc: "The back curve radius of the lens"
  },
  lenscenterthickness: {
      name: "Center Thickness",
      desc: "The center thickness of the lens"
  },
  lensbowldiameter: {
      name: "Bowl Diameter",
      desc: "The bowl diameter of the lens"
  },
  lenscarriercurve: {
      name: "Carrier Curve",
      desc: "The carrier curve of the lens"
  },
  lenscarriercurveradius: {
      name: "Carrier Curve Radius",
      desc: "The carrier curve radius of the lens"
  },
  lenssegmentthickness: {
      name: "Segment Thickness",
      shortname: "Seg Thickness",
      desc: "The segment thickness of the lens"
  },
  lenssagreading: {
      name: "Sag Reading",
      desc: "The sag reading of the lens"
  },
  lenspolarizedcurve: {
      name: "Polarized Curve",
      desc: "The polarized curve of the lens"
  },
  lenspolarizedcurveradius: {
      name: "Polarized Curve Radius",
      desc: "The polarized curve radius of the lens"
  },
  lenspolarizedmmfromfront: {
      name: "Polarized Curve MM From Front",
      desc: "The polarized curve mm from front of the lens"
  },
  lensblkd: {
      name: "BLKD",
      desc: "The blkd of the lens"
  },
  lensrngh: {
      name: "RNGH",
      desc: "The rngh of the lens"
  },
  lenslblhi: {
      name: "LBLHI",
      desc: "The lblhi of the lens"
  },
  lensblkdm: {
      name: "BLKDM",
      desc: "The blkdm of the lens"
  },
  lensblkcehi: {
      name: "BLKCEHI",
      desc: "The blkcehi of the lens"
  },
  lensblkrd: {
      name: "BLKRD",
      desc: "The blkrd of the lens"
  },
  lenskprvm: {
      name: "KPRVM",
      desc: "The kprvm of the lens"
  },
  lenskprva: {
      name: "KPRVA",
      desc: "The kprva of the lens"
  },
  lens_warehouse_data: {
      name: "Warehouse",
      desc: "Warehouse records for this lens"
  },
=begin
      lens_transactions: {
          name: "Transactions",
          desc: "Transactions that involve this lens"
      },
      po_lines: {
          name: "Purchase Order Lines",
          desc: "Purchase order lines that involve this lens"
      },
=end
  })
end
