# == Schema Information
#
# Table name: lens_transactions
#
#  id                      :bigint           not null, primary key
#  lmssource               :string           not null
#  stxcomputedinstock      :integer
#  stxcomputedinstockleft  :integer
#  stxcomputedinstockright :integer
#  stxcount                :integer
#  stxcountleft            :integer
#  stxcountright           :integer
#  stxdatetime             :datetime         not null
#  stxdescription          :string
#  stxemployeecode         :string
#  stxlensstocknum         :string
#  stxlenswarehouseid      :string
#  stxopc                  :string
#  stxordernum             :string
#  stxqty                  :integer
#  stxqtyleft              :integer
#  stxqtyright             :integer
#  stxtotalcost            :decimal(12, 3)
#  stxtotalvendorcost      :decimal(12, 3)
#  stxtransactioncode      :string
#  stxtransactioncodedesc  :string
#  stxtransactiontype      :string
#  stxtransactiontypedesc  :string
#  stxunitcost             :decimal(12, 3)
#  stxvendorcost           :decimal(12, 3)
#  warehouse_link          :string
#  created_at              :datetime         not null
#  updated_at              :datetime         not null
#  lens_id                 :bigint
#
# Indexes
#
#  index_lens_transactions_on_lens_id             (lens_id)
#  index_lens_transactions_on_stxdescription      (stxdescription)
#  index_lens_transactions_on_stxlensstocknum     (stxlensstocknum)
#  index_lens_transactions_on_stxlenswarehouseid  (stxlenswarehouseid)
#  index_lens_transactions_on_stxopc              (stxopc)
#  index_lens_transactions_on_stxordernum         (stxordernum)
#  index_lens_transactions_on_stxtransactioncode  (stxtransactioncode)
#  index_lens_transactions_on_stxtransactiontype  (stxtransactiontype)
#  index_lens_transactions_on_warehouse_link      (warehouse_link)
#  lens_transaction_unique_key                    (stxdatetime,stxlenswarehouseid,lmssource) UNIQUE
#
# Foreign Keys
#
#  fk_rails_...  (lens_id => lenses.id) ON DELETE => cascade ON UPDATE => cascade
#
class LensTransaction < ApplicationRecord
  belongs_to :lens, optional: true
  has_one :lens_warehouse_datum,
          primary_key: :warehouse_link,
          foreign_key: :stx_link
  has_one :order,
          primary_key: :stxordernum,
          foreign_key: :ordernum
  after_save :update_po_receive_date

  class << self
    def import_from_hash(h)
      h.merge!({"lens" => (h.delete("lenstechnical") || {})
                              .merge((h.delete("lenscharacteristics") || {}))
                              .merge((h.delete("stxlenscharacteristics") || {}))
                              .merge((h.delete("stxlenstechnical") || {}))})
      h.merge!(h.delete("stx") || {})
      h['stxlensstocknum'] ||= h.delete('lensstocknum')
      h['stxlenswarehouseid'] ||= h.delete('lenswarehouseid')
      if h["lens"] == {}
        h.delete("lens")
      else
        h["lens"].merge!({"lmssource" => h["lmssource"]})
        %w(lensdescription lensmanufacturercode lensmanufacturerdesc lensstylecode lensstyledesc lensstyletype lensmaterialcode lensmaterialdesc lenstintcode lenstintdesc
lenstreatcode lenstreatdesc lensclass lenssegwidth lenssegdepth lenssphere lenscylinder lensbase lensadd lensupperadd lensdiameter lensthicknesscode lensopc lensopcright lensopcleft).each do |k|
          h["lens"][k] ||= h["lens"].delete("stx#{k}")
        end
      end
      super(h)
    end

    def conflict_target
      [:stxdatetime, :stxlenswarehouseid, :lmssource]
    end
  end

  def update_po_receive_date
    if self.stxtransactioncode == 'C' &&
        self.stxdescription &&
        self.stxdescription != '' &&
        (po = PoHeader.where(ponum: self.stxdescription).where(PoHeader.get_arel('poreceivedate', false).lt(self.stxdatetime).or(PoHeader.get_arel('poreceivedate', false).eq(nil))).first)
      Rails.logger.info { "Updating po receive date #{self.stxdescription} from #{po[:poreceivedate]} to #{self.stxdatetime}" }
      po.update_columns(poreceivedate: self.stxdatetime) #update columsn skips the po's own save trigger
    end
  end

  compute_on_save(:warehouse_link) do |trx|
    "#{trx[:stxlenswarehouseid]}#{trx[:lens_id]}"
  end

  reportable("lens_transaction", "Lens Transactions", "Report on lens transactions", {
=begin
      lmssource: {
          name: "Lab Management System",
          shortname: "LMS",
          desc: "The lab management system that produced the transaction"
      },
=end
  stxopc: {
      name: "OPC",
      desc: "The opc of the lens this transaction is attached to"
  },
  stxdatetime: {
      name: "Date",
      desc: "The time and date this transaction was performed on"
  },
  stxtransactiontype: {
      name: "Type",
      desc: "The type of transaction performed"
  },
  stxtransactiontypedesc: {
      name: "Type Description",
      shortname: "Type Desc",
      desc: "The type of transaction performed"
  },
  stxtransactioncode: {
      name: "Code",
      desc: "The code of the transaction performed"
  },
  stxtransactioncodedesc: {
      name: "Code Description",
      shortname: "Code Desc",
      desc: "The description of the code of the transaction performed"
  },
  stxdescription: {
      name: "Description",
      shortname: "Desc",
      desc: "The description of the transaction performed"
  },
  stxqty: {
      name: "Quantity",
      shortname: "Qty",
      desc: "The quantity of the transaction performed"
  },
  stxqtyright: {
      name: "Right Quantity",
      shortname: "R.Qty",
      desc: "The right quantity of the transaction performed"
  },
  stxqtyleft: {
      name: "Left Quantity",
      shortname: "L.Qty",
      desc: "The left quantity of the transaction performed"
  },
  stxunitcost: {
      name: "Unit Cost",
      desc: "The unit cost of the transaction performed"
  },
  stxcount: {
      name: "Count",
      desc: "The count of the transaction performed"
  },
  stxcountright: {
      name: "Right Count",
      shortname: "R.Count",
      desc: "The right count of the transaction performed"
  },
  stxcountleft: {
      name: "Left Count",
      shortname: "L.Count",
      desc: "The right count of the transaction performed"
  },
  stxemployeecode: {
      name: "Employee Code",
      shortname: "Employee",
      desc: "The employee that performed the transaction"
  },
  stxordernum: {
      name: "Order Number",
      shortname: "Order",
      desc: "The order attached to the transaction"
  },
  stxlensstocknum: {
      name: "Stock Number",
      shortname: "Stock Num",
      desc: "The stock number attached to the transaction"
  },
  stxlenswarehouseid: {
      name: "Warehouse ID",
      shortname: "Warehouse",
      desc: "The warehouse number attached to the transaction"
  },
  stxtotalcost: {
      name: "Total Cost",
      shortname: "Cost",
      desc: "The total cost attached to the transaction"
  },
  stxcomputedinstock: {
      name: "Computed In Stock",
      desc: "The computed in stock attached to the transaction"
  },
  stxcomputedinstockright: {
      name: "Right Computed In Stock",
      shortname: "R.Computed In Stock",
      desc: "The right computed in stock attached to the transaction"
  },
  stxcomputedinstockleft: {
      name: "Left Computed In Stock",
      shortname: "L.Computed In Stock",
      desc: "The left computed in stock attached to the transaction"
  },
  stxvendorcost: {
      name: "Vendor Cost",
      shortname: "V.Cost",
      desc: "The vendor cost attached to the transaction"
  },
  stxtotalvendorcost: {
      name: "Total Vendor Cost",
      shortname: "Total V.Cost",
      desc: "The total vendor cost attached to the transaction"
  },
  lens: {
      name: "Lens",
      desc: "The lens this transaction is attached to"
  },
  order: {
      name: "Order",
      desc: "The orders this transaction is attached to (if any)"
  },
  lens_warehouse_datum: {
      name: "Current Warehouse Info",
      shortname: "Warehouse",
      desc: "The warehouse data of the lens this transaction is attached to"
  },
  })
end
