# == Schema Information
#
# Table name: lens_warehouse_data
#
#  id                             :bigint           not null, primary key
#  lensaveragecost                :decimal(12, 3)
#  lensaveragedailyusage          :float
#  lensaveragemonthlyusage        :float
#  lensbin                        :string
#  lensdatewanteddays             :integer
#  lenshistorymonths              :integer
#  lensmaximumqty                 :integer
#  lensmaxsupplydays              :integer
#  lensoverstockamount            :decimal(12, 3)
#  lensoverstockflag              :boolean
#  lensoverstockqty               :integer
#  lenspriority                   :integer
#  lensqtyallocated               :integer
#  lensqtyallocatedleft           :integer
#  lensqtyallocatedright          :integer
#  lensqtyeoq                     :float
#  lensqtyinproc                  :integer
#  lensqtyinprocleft              :integer
#  lensqtyinprocright             :integer
#  lensqtyinstock                 :integer
#  lensqtyinstockleft             :integer
#  lensqtyinstockright            :integer
#  lensqtyonorder                 :integer
#  lensqtyonorderleft             :integer
#  lensqtyonorderright            :integer
#  lensqtyonshelf                 :integer
#  lensqtyonshelfleft             :integer
#  lensqtyonshelfright            :integer
#  lensqtyusedsincelastorder      :integer
#  lensqtyusedsincelastorderleft  :integer
#  lensqtyusedsincelastorderright :integer
#  lensreorderfactor              :float
#  lensreorderlevel               :integer
#  lensreordermethod              :string
#  lenssafetydays                 :integer
#  lensshipdays                   :integer
#  lensstockitem                  :boolean
#  lensstocknum                   :string
#  lenstotalaveragecost           :decimal(12, 3)
#  lenstotalstandardcost          :decimal(12, 3)
#  lensvalallocated               :decimal(12, 3)
#  lensvalinproc                  :decimal(12, 3)
#  lensvalinstock                 :decimal(12, 3)
#  lensvalonorder                 :decimal(12, 3)
#  lensvalonshelf                 :decimal(12, 3)
#  lensvalusedsincelastorder      :decimal(12, 3)
#  lensvendor                     :string
#  lensvendorcost                 :decimal(12, 3)
#  lenswarehouseid                :string
#  stx_link                       :string
#  created_at                     :datetime         not null
#  updated_at                     :datetime         not null
#  lens_id                        :bigint
#
# Indexes
#
#  index_lens_warehouse_data_on_lens_id                           (lens_id)
#  index_lens_warehouse_data_on_lensaveragemonthlyusage           (lensaveragemonthlyusage)
#  index_lens_warehouse_data_on_lensstocknum_and_lenswarehouseid  (lensstocknum,lenswarehouseid)
#  index_lens_warehouse_data_on_lenswarehouseid                   (lenswarehouseid)
#  index_lens_warehouse_data_on_stx_link                          (stx_link)
#
# Foreign Keys
#
#  fk_rails_...  (lens_id => lenses.id) ON DELETE => cascade ON UPDATE => cascade
#
class LensWarehouseDatum < ApplicationRecord
  belongs_to :lens, optional: true
  has_many :lens_transaction,
           primary_key: :stx_link,
           foreign_key: :warehouse_link

  class << self
    def conflict_target
      [:lenswarehouseid, :lens_id]
    end
  end

  compute_on_save(:stx_link) do |trx|
    "#{trx[:lenswarehouseid]}#{trx[:lens_id]}"
  end

  compute_on_save(:lensvalinstock) do |trx|
    (BigDecimal(trx[:lensqtyinstock] || 0) + BigDecimal(trx[:lensqtyinstockright] || 0) + BigDecimal(trx[:lensqtyinstockleft] || 0)) * BigDecimal(trx[:lensvendorcost] || 0)
  end

  compute_on_save(:lensvalonorder) do |trx|
    (BigDecimal(trx[:lensqtyonorder] || 0) + BigDecimal(trx[:lensqtyonorderright] || 0) + BigDecimal(trx[:lensqtyonorderleft] || 0)) * BigDecimal(trx[:lensvendorcost] || 0)
  end

  compute_on_save(:lensvalallocated) do |trx|
    (BigDecimal(trx[:lensqtyallocated] || 0) + BigDecimal(trx[:lensqtyallocatedright] || 0) + BigDecimal(trx[:lensqtyallocatedleft] || 0)) * BigDecimal(trx[:lensvendorcost] || 0)
  end

  compute_on_save(:lensvalinproc) do |trx|
    (BigDecimal(trx[:lensqtyinproc] || 0) + BigDecimal(trx[:lensqtyinprocright] || 0) + BigDecimal(trx[:lensqtyinprocleft] || 0)) * BigDecimal(trx[:lensvendorcost] || 0)
  end

  compute_on_save(:lensvalusedsincelastorder) do |trx|
    (BigDecimal(trx[:lensqtyusedsincelastorder] || 0) + BigDecimal(trx[:lensqtyusedsincelastorderright] || 0) + BigDecimal(trx[:lensqtyusedsincelastorderleft] || 0)) * BigDecimal(trx[:lensvendorcost] || 0)
  end

  compute_on_save(:lensvalonshelf) do |trx|
    (BigDecimal(trx[:lensqtyonshelf] || 0) + BigDecimal(trx[:lensqtyonshelfright] || 0) + BigDecimal(trx[:lensqtyonshelfleft] || 0)) * BigDecimal(trx[:lensvendorcost] || 0)
  end

  reportable("lens_warehouse_data", "Warehouse Contents", "Report on lens warehouse data", {
      lenswarehouseid: {
          name: "Warehouse ID",
          shortname: "Warehouse",
          desc: "The warehouse the lens is kept at"
      },
      lensstocknum: {
          name: "Lens Stock Number",
          shortname: "Stock Num",
          desc: "The warehouse the lens is kept at"
      },
      lensbin: {
          name: "Lens Bin",
          shortname: "Bin",
          desc: "The physical location of the lens"
      },
      lensstockitem: {
          name: "Stock Item?",
          desc: "Whether the lens is a stock item or not"
      },
      lenspriority: {
          name: "Lens Priority",
          shortname: "Priority",
          desc: "Whether the lens is a stock item or not"
      },
      lensdatewanteddays: {
          name: "Date Wanted Days",
          desc: "The number of days applicable for date wanted for the lens"
      },
      lensreordermethod: {
          name: "Lens Reorder Method",
          shortname: "Reorder Method",
          desc: "The reorder method for the lens"
      },
      lenssafetydays: {
          name: "Lens Safety Days",
          shortname: "Safety Days",
          desc: "The safety days for the lens"
      },
      lensshipdays: {
          name: "Lens Ship Days",
          shortname: "Ship Days",
          desc: "The ship days for the lens"
      },
      lensmaxsupplydays: {
          name: "Lens Max Supply Days",
          shortname: "Max Supply Days",
          desc: "The max supply days for the lens"
      },
      lenshistorymonths: {
          name: "Lens History Months",
          shortname: "History Months",
          desc: "The history months for the lens"
      },
      lensreorderfactor: {
          name: "Lens Reorder Factor",
          shortname: "Reorder Factor",
          desc: "The reorder factor for the lens"
      },
      lensreorderlevel: {
          name: "Lens Reorder Level",
          shortname: "Reorder Level",
          desc: "The reorder level for the lens"
      },
      lensmaximumqty: {
          name: "Lens Maximum Quantity",
          shortname: "Max Qty",
          desc: "The maximum quantity for the lens"
      },
      lensqtyinstock: {
          name: "Lens Quantity In Stock",
          shortname: "In Stock",
          desc: "The current in-stock quantity for the lens"
      },
      lensqtyinstockright: {
          name: "Right Lens Quantity In Stock",
          shortname: "R.In Stock",
          desc: "The current in-stock quantity for the right lens"
      },
      lensqtyinstockleft: {
          name: "Left Lens Quantity In Stock",
          shortname: "L.In Stock",
          desc: "The current in-stock quantity for the left lens"
      },
      lensvalinstock: {
          name: "Value In Stock",
          shortname: "Val In Stock",
          desc: "Quantity in stock (overall, left/right/achiral) * Vendor Cost"
      },
      lensqtyeoq: {
          name: "Lens Quantity EOQ",
          shortname: "EOQ",
          desc: "The eoq quantity for the lens"
      },
      lensvendor: {
          name: "Lens Vendor",
          shortname: "Vendor",
          desc: "The vendor that provides the lens"
      },
      lensvendorcost: {
          name: "Lens Vendor Cost",
          shortname: "Vendor Cost",
          desc: "The vendor cost for the lens"
      },
      lensaveragecost: {
          name: "Lens Average Cost",
          shortname: "Average Cost",
          desc: "The average cost for the lens"
      },
      lensaveragemonthlyusage: {
          name: "Lens Average Monthly Usage",
          shortname: "Average Monthly Usage",
          desc: "The average monthly usage for the lens"
      },
      lensaveragedailyusage: {
          name: "Lens Average Daily Usage",
          shortname: "Average Daily Usage",
          desc: "The average daily usage for the lens"
      },
      lensqtyonorder: {
          name: "Lens Quantity On Order",
          shortname: "On Order",
          desc: "The quantity on order for the lens"
      },
      lensqtyonorderright: {
          name: "Right Lens Quantity On Order",
          shortname: "R.On Order",
          desc: "The quantity on order for the right lens"
      },
      lensqtyonorderleft: {
          name: "Left Lens Quantity On Order",
          shortname: "L.On Order",
          desc: "The quantity on order for the left lens"
      },
      lensvalonorder: {
          name: "Value On Order",
          shortname: "On Order",
          desc: "Quantity on order (overall, left/right/achiral) * Vendor Cost"
      },
      lensqtyallocated: {
          name: "Lens Quantity Allocated",
          shortname: "Allocated",
          desc: "The allocated quantity for the lens"
      },
      lensqtyallocatedright: {
          name: "Right Lens Quantity Allocated",
          shortname: "R.Allocated",
          desc: "The allocated quantity for the right lens"
      },
      lensqtyallocatedleft: {
          name: "Left Lens Quantity Allocated",
          shortname: "L.Allocated",
          desc: "The allocated quantity for the left lens"
      },
      lensvalallocated: {
          name: "Value Allocated",
          shortname: "Allocated",
          desc: "Quantity allocated (overall, left/right/achiral) * Vendor Cost"
      },
      lensqtyinproc: {
          name: "Lens Quantity In Process",
          shortname: "In Process",
          desc: "The in-process quantity for the lens"
      },
      lensqtyinprocright: {
          name: "Right Lens Quantity In Process",
          shortname: "R.In Process",
          desc: "The in-process quantity for the right lens"
      },
      lensqtyinprocleft: {
          name: "Left Lens Quantity In Process",
          shortname: "L.In Process",
          desc: "The in-process quantity for the left lens"
      },
      lensvalinproc: {
          name: "Value In Process",
          shortname: "In Process",
          desc: "Quantity in process (overall, left/right/achiral) * Vendor Cost"
      },
      lensqtyusedsincelastorder: {
          name: "Lens Quantity Used Since Last Order",
          shortname: "Used Since Last Order",
          desc: "The number of lenses used since the last order"
      },
      lensqtyusedsincelastorderright: {
          name: "Right Lens Quantity Used Since Last Order",
          shortname: "R.Used Since Last Order",
          desc: "The number of right lenses used since the last order"
      },
      lensqtyusedsincelastorderleft: {
          name: "Left Lens Quantity Used Since Last Order",
          shortname: "L.Used Since Last Order",
          desc: "The number of left lenses used since the last order"
      },
      lensvalusedsincelastorder: {
          name: "Value Used Since Last Order",
          shortname: "Used Since Last Order",
          desc: "Quantity used since last order (overall, left/right/achiral) * Vendor Cost"
      },
      lenstotalaveragecost: {
          name: "Total Average Cost",
          desc: "The total average cost for the lens"
      },
      lenstotalstandardcost: {
          name: "Total Standard Cost",
          desc: "The total standard cost for the lens"
      },
      lensqtyonshelf: {
          name: "Lens Quantity On Shelf",
          shortname: "On Shelf",
          desc: "The quantity on shelf for the lens"
      },
      lensqtyonshelfright: {
          name: "Right Lens Quantity On Shelf",
          shortname: "R.On Shelf",
          desc: "The quantity on shelf for the right lens"
      },
      lensqtyonshelfleft: {
          name: "Left Lens Quantity On Shelf",
          shortname: "L.On Shelf",
          desc: "The quantity on shelf for the left lens"
      },
      lensvalonshelf: {
          name: "Value On Shelf",
          shortname: "On Shelf",
          desc: "Quantity used since last order (overall, left/right/achiral) * Vendor Cost"
      },
      lensoverstockflag: {
          name: "Lens Overstock?",
          shortname: "Overstock",
          desc: "Indicates whether the lens is overstock or not"
      },
      lensoverstockqty: {
          name: "Lens Overstock Quantity",
          shortname: "Overstock Qty",
          desc: "The overstock quantity for the lens"
      },
      lensoverstockamount: {
          name: "Lens Overstock Amount",
          shortname: "Overstock Amount",
          desc: "The overstock amount for the lens"
      },
      lens: {
          name: "Lens",
          desc: "The lens being inventoried"
      },
  })
end
