# == Schema Information
#
# Table name: po_headers
#
#  id                :bigint           not null, primary key
#  labnum            :string           not null
#  lmssource         :string           not null
#  poamount          :decimal(12, 2)
#  poclosedate       :date
#  podate            :date
#  ponum             :string           not null
#  poopcconfirmation :string
#  poopcfilename     :string
#  poqty             :integer
#  poreceivedate     :datetime
#  poreordermethod   :string
#  postatus          :string
#  pototal           :decimal(12, 2)
#  potransmitted     :boolean
#  potype            :string
#  povendorid        :string
#  povendorname      :string
#  created_at        :datetime         not null
#  updated_at        :datetime         not null
#
# Indexes
#
#  index_po_headers_on_labnum                          (labnum)
#  index_po_headers_on_poclosedate                     (poclosedate)
#  index_po_headers_on_podate                          (podate)
#  index_po_headers_on_ponum_and_labnum_and_lmssource  (ponum,labnum,lmssource) UNIQUE
#  index_po_headers_on_poreceivedate                   (poreceivedate)
#  index_po_headers_on_postatus                        (postatus)
#  index_po_headers_on_potype                          (potype)
#  index_po_headers_on_povendorid                      (povendorid)
#
class PoHeader < ApplicationRecord
  has_many :po_lines, dependent: :destroy

  class << self
    def import_from_hash(h)
      h.merge!({"po_lines" => clean_multi_relation(h.delete("polines"))})
      h["labnum"] ||= h.delete('polabnum')
      h["po_lines"]&.each do |l|
        l["lmssource"] = h["lmssource"]
      end
      super(h)
    end

    def conflict_target
      [:ponum, :labnum, :lmssource]
    end

    def destroy_old_on_import?(assoc)
      true
    end
  end

  compute_on_save(:poqty) do |po|
    po.po_lines.reduce(0) do |sum, line|
      sum + (line[:lensqtyreceived] || 0).to_i + (line[:lensqtyreceivedright] || 0).to_i + (line[:lensqtyreceivedleft] || 0).to_i
    end
  end

  compute_on_save(:poamount) do |po|
    po.po_lines.reduce(0) do |sum, line|
      line_qty = (line[:lensqtyreceived] || 0).to_i + (line[:lensqtyreceivedright] || 0).to_i + (line[:lensqtyreceivedleft] || 0).to_i
      line_amount = (line_qty * BigDecimal(line[:lensunitprice] || 0))
      line_amount = line_amount.round(3) if (line_amount - line_amount.round(0)) >= 0.4999 # try to avoid floating point nonsense
      sum + line_amount.round(2)
    end
  end

  compute_on_save(:poreceivedate) do |po|
    resolved = LensTransaction.where(stxtransactioncode: 'C', stxdescription: po[:ponum]).order(stxdatetime: :desc).limit(1).pluck(:stxdatetime)&.[](0)
    if resolved && po[:poreceivedate]
      if resolved >= po[:poreceivedate]
        resolved
      else
        po[:poreceivedate]
      end
    else
      resolved || po[:poreceivedate]
    end
  end

  reportable("po_header", "Purchase Order Header", "Report on purchase orders", {
=begin
      lmssource: {
          name: "Lab Management System",
          shortname: "LMS",
          desc: "The lab management system that produced the purchase order"
      },
=end
  labnum: {
      name: "Lab Number",
      shortname: "Lab",
      desc: "The lab that produced the purchase order"
  },
  ponum: {
      name: "PO Number",
      shortname: "PO Num",
      desc: "The identifying number for the purchase order"
  },
  postatus: {
      name: "Status",
      desc: "The status for the purchase order"
  },
  povendorid: {
      name: "Vendor ID",
      shortname: "Vendor",
      desc: "The vendor id for the purchase order"
  },
  povendorname: {
      name: "Vendor Name",
      shortname: "Vendor",
      desc: "The vendor name for the purchase order"
  },
  podate: {
      name: "Date",
      desc: "The date for the purchase order"
  },
  pototal: {
      name: "Total",
      desc: "The total for the purchase order"
  },
  poreordermethod: {
      name: "Reorder Method",
      desc: "The reorder method for the purchase order"
  },
  potype: {
      name: "Type",
      desc: "The type for the purchase order"
  },
  poopcfilename: {
      name: "OPC Filename",
      desc: "The opc filename for the purchase order"
  },
  poopcconfirmation: {
      name: "OPC Confirmation",
      desc: "The opc confirmation for the purchase order"
  },
  potransmitted: {
      name: "Transmitted?",
      shortname: "Trans",
      desc: "The transmission for the purchase order"
  },
  poclosedate: {
      name: "Close Date",
      shortname: "Close",
      desc: "The close date for the purchase order"
  },
  poreceivedate: {
      name: "Receive Date",
      shortname: "Receive",
      desc: "The receive date for the purchase order"
  },
  poqty: {
      name: "Total Quantity",
      shortname: "Qty",
      desc: "The total lens quantity for the purchase order"
  },
  poamount: {
      name: "Total Amount",
      shortname: "Amount",
      desc: "The total lens quantity times the lens unit price for the purchase order"
  },
  po_lines: {
      name: "Lines",
      desc: "The line items for the purchase order"
  },
  })
end
